<?php
namespace TekNutri\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\DBAL\Connection;
use Exception;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Sincronizacao {

	protected $connection;
	protected $fileUploader;
	protected $rotaEnvia;
	protected $rotaReenvia;
	protected $rotaPreparaAmbiente;
    
    protected $nrOrg;
    protected $cdOperador;
    
    public function __construct($connection, $fileUploader,  $baseSinc){
        $this->connection    = $connection;
        $this->fileUploader  = $fileUploader;
		$this->rotaEnvia     = $baseSinc."recebeDados";
		$this->rotaReenvia   = $baseSinc."reenviaDados";
		$this->rotaPreparaAmbiente = $baseSinc."preparaAmbiente";
	}
	
	
	public function sincronizar($request, $response){
	    //$this->excluiDir("/home/developer/workfolder/tecfood-teknutri/backend/service/Response");die;
	    date_default_timezone_set('America/Sao_Paulo');
        $orgRemessas = [];
    	$rows = $request->getDataSet()->getRows();
    	
    	$this->nrOrg = $rows[0]['NRORG'];
    	if(array_key_exists('CDOPERADOR', $rows[0])) $this->cdOperador = $rows[0]['CDOPERADOR'];
    	else  $this->cdOperador = "000125241174"; // produto@tecfood.com para sincronização automática
    	
    	$resultAmb = $this->verificaAmbiente($rows, $request, $response);
        $rows = $resultAmb[0];
        $orgAmb = $resultAmb[1];

	    $resultPendencias = $this->verificarPendencias($rows, $request, $response);
	    $rows = $resultPendencias[0];
	    $orgPendentes = $resultPendencias[1];
        foreach($rows as $row){
            if($row['PENDENTE'] == 'S' ) continue;
            try{
                do{
                    do{
                        $nrorgint = $row["NRORGINT"];
                        $nmOrgInt = $row["NMORGANIZACAO"];
                        $cdRemessa = $this->gerarCdRemessa();
                        $insercao = $this->inserirRemessa($cdRemessa, $nrorgint);
                        if ($insercao !== true) {
                            $response->setError(new Error($insercao, 1));
                            return false;
                        }
                        $produtosLiberados = $this->listarProdutosLiberados($nrorgint);
                        $arrayProdutos = $this->montarArrayProdutos($produtosLiberados, $nrorgint, $cdRemessa, true);
                        $url = $row['URLINTEGRAORG'].$this->rotaEnvia;
                        $qtProdutosRecebidos = $this->realizarIntegracao($arrayProdutos, $url);
                        $this->atualizaStatusRemessa($cdRemessa, $nrorgint);
                    }while(count($produtosLiberados) == 100);
                }while($qtProdutosRecebidos == 1000); 
            }catch(\Exception $e){
                $msg = $this->trataErros($request, $response, $e->getMessage(), $cdRemessa, $row);     
                $atualizacao = $this->inserirMensagemErro($msg, $cdRemessa);
                if ($atualizacao !== true) {
                    $response->setError(new Error($atualizacao, 1));
                    return false;
                }
                array_push($orgRemessas,
                [ "NMORG"   => $nmOrgInt,
                  "ERROR"   => $msg
                ]);
                
            }
        }    
	   
	   $organizacoes = array_merge($orgAmb, $orgPendentes, $orgRemessas);
	   if(!$organizacoes) $organizacoes = [];
       $response->addDataSet(new DataSet("organizacoes", $organizacoes));
    }
    
    private function trataErros($request, $response, $msgErro, $cdRemessa, $row) {
        try {
            $msgRetorno = $msgErro;
            if(strpos($msgErro, "FKPLA00030PRODUTO") !== false){
                $msgRetorno = "Produto excluído no cliente";
                
                //tratamento para pegar informacoes do produto
                $msgErro = str_replace('"', '', $msgErro);
                preg_match_all('/\[(?:[^\[\]]|(?R))*\]/', $msgErro, $arrayError); // pega tudo dentro de array
                $stringError = preg_replace('/(\[|\]| )/', '', $arrayError[0][0]); // retira colchetes e espacos
                $produto = explode(',', $stringError);
                
                if (!empty($produto)) {
                    $cdproduto = $produto[0];
                    // atualiza data de envio
                    $sql = "UPDATE SIT_PRODUTOINT
                               SET DTENVIO = :DTENVIO,
                                   DTULTATU = :DTULTATU,
                                   CDOPERULTATU = :CDOPERULTATU
                             WHERE DTLIBERACAO IS NOT NULL
                               AND CDPRODUTOINT = :CDPRODUTOINT
                               AND NRORGINT = :NRORGINT";
                    
                    $stmt = $this->connection->prepare($sql);
                    $params = array(
                        "DTENVIO"      => date('d/m/Y H:i:s', time()),
                        "DTULTATU"     => date('d/m/Y H:i:s', time()),
                        "CDOPERULTATU" => $this->cdOperador,
                        "CDPRODUTOINT" => $cdproduto,
                        "NRORGINT"     => $row['NRORGINT']
                    );
                    $stmt->execute($params);
                    
                    // atualiza log
                    $sql = "UPDATE SIT_LOGPROD
                            SET DSOBSERVACAO = :DSOBSERVACAO,
                                DTULTATU = :DTULTATU,
                                CDOPERULTATU = :CDOPERULTATU
                            WHERE CDPRODUTO = :CDPRODUTO
                            AND NRREMESSA = :NRREMESSA
                            AND NRORGINT = :NRORGINT";
                    
                    $stmt = $this->connection->prepare($sql);
                    $params = array(
                        "DSOBSERVACAO" => $msgRetorno,
                        "DTULTATU"     => date('d/m/Y H:i:s', time()),
                        "CDOPERULTATU" => $this->cdOperador,
                        "CDPRODUTO"    => $cdproduto,
                        "NRREMESSA"    => $cdRemessa,
                        "NRORGINT"     => $row['NRORGINT']
                    );
                    $stmt->execute($params);
                    
                    // continua com a sincronização
                    $this->sincronizar($request, $response);
                }
            } else if(strpos($msgErro, "403 Forbidden") !== false || $msgErro == 'URL_NAO_ENCONTRADA' || strpos($msgErro, "404 Not Found") !== false) $msgRetorno = "Servidor do cliente não encontrado (".$row['URLINTEGRAORG']."). Verificar com o cliente se o produto está disponível no endereço cadastrado na tela de 'Serviços Contratados'";
            else if(strpos($msgErro, "HTTP 0") !== false) $msgRetorno = "Solicitação demorou a responder. Firewall do cliente está impedindo acesso no momento da sincronização. Verificar com o cliente se algum bloqueia para receber requisição da Teknisa";
            else if(strpos($msgErro, "401 Unauthorized") !== false) $msgRetorno = "Acesso não autorizado. Verificar se o cliente autorizou o acesso da Teknisa";
            
            return $msgRetorno;
        }catch(\Exception $e){
            throw new \Exception($e->getMessage()); 
        }
    }
	
	private function verificaAmbiente($rows, $request, $response){
	    
	    $newRows = [];
	    $orgPendentes = [];
        $infoNutr = $this->retornaArrayNutr();
        $paramgeral = $this->listarParamgeral();
        $arrayNutr = array(
            "NRORGINT"  => '',
            "PARAMGERAL"=> $paramgeral[0],
            "INFONUTR"  => $infoNutr
        );
        
	    foreach($rows as $row){
	        $row['PENDENTE'] = 'N';
	        try{
	            $sitContrato = $this->listarInfOrganizacao($row['NRORGINT']);
	            if($sitContrato[0]['IDCONSNUT'] == 'S'){
    	            $remessa = $this->primeiraSincronizacao($row['NRORGINT']);
        	        if(count($remessa) == 0){
        	          $arrayNutr['NRORGINT'] = $row['NRORGINT'];
        	          $result = $this->preparaAmbiente($row, $arrayNutr, $response);
    
                    }else if($remessa[0]['IDSITREMESSA'] == 'N') $row['PENDENTE'] = 'S';
	                
	            }
	        }catch(\Exception $e){
	            $msg = $this->trataErros($request, $response, $e->getMessage(), $cdRemessa, $row);
                $row['PENDENTE'] = 'S';
	            array_push($orgPendentes,
                [ "NMORG"   => $row['NMORGANIZACAO'],
                  "ERROR"   => $msg
                ]);
	        }
	        array_push($newRows,$row);
	    }
	    return [$newRows, $orgPendentes];
	}
	
	private function retornaArrayNutr(){
	    try{
            $query = "SELECT GRUPNUTR.CDGRUPNUTR, GRUPNUTR.NMGRUPNUTR, NUTRIENTE.CDNUTRIENTE, 
                                        NUTRIENTE.NMNUTRIENTE, NUTRIENTE.NMREDUZIDO, NUTRIENTE.IDTIPONUTR,
                                        UNIDMEDIDA.SGUNIDADE, UNIDMEDIDA.DSUNIDADE,
                                        NUTRIENTE.CDEXPARQTXTVND,NUTRIENTE.IDCALORIAS, 
                                        NUTRIENTE.IDNUTRRELEV,NUTRIENTE.VRFATORNUTR, NUTRIENTE.IDKCAL,
                                        NUTRIENTE.NRORDEMNUTR, NUTRIENTE.VRLIMDIARIONT,
                                        GRUPNUTR.NRORG
                                  FROM GRUPNUTR
                                  LEFT JOIN NUTRIENTE ON NUTRIENTE.CDGRUPNUTR = GRUPNUTR.CDGRUPNUTR
                                  LEFT JOIN UNIDMEDIDA ON UNIDMEDIDA.SGUNIDADE = NUTRIENTE.SGUNIDADE
                                  WHERE GRUPNUTR.NRORG =:NRORG
                                  ORDER BY CDGRUPNUTR, CDNUTRIENTE";
            $stmt = $this->connection->prepare($query);
            $params = array(
                "NRORG"  => $this->nrOrg
            );
            $stmt->execute($params);
            $infoNutr = $stmt->fetchAll(\PDO::FETCH_ASSOC);
            return $infoNutr;
	    }catch(\Exception $e){
	        throw new \Exception($e->getMessage());
	    }
	    
	}
	
	private function primeiraSincronizacao($nrorgint){
	    $result = true;
	    $query = "SELECT * 
	                FROM (SELECT * 
	                        FROM REMPRODNUTR
                           WHERE NRORGINT =:NRORGINT
    	                   ORDER BY NRREMESSA ASC
    	                 )
                   WHERE ROWNUM = 1";
    	    
        $stmt = $this->connection->prepare($query);
        $params = array(
            "NRORGINT" =>  $nrorgint
        );
        $stmt->execute($params);
        return $remessa = $stmt->fetchAll(\PDO::FETCH_ASSOC);
	}
	
	public function primeiraRemessa($request, $response){
	    $row = $request->getRow();
	    $remessa = $this->primeiraSincronizacao($row['NRORGINT'])[0];
	    $response->addDataSet(new DataSet("remessa", $remessa));
	}
	
	private function preparaAmbiente($row, $arrayNutr, $response){
    	try{

            $cdRemessa = $this->gerarCdRemessa();
            $nrorgint = $row['NRORGINT'];
            
            $insercao = $this->inserirRemessa($cdRemessa, $nrorgint);
            if ($insercao !== true) {
                $response->setError(new Error($insercao, 1));
                return false;
            }
            
    	    $url = $row['URLINTEGRAORG'].$this->rotaPreparaAmbiente;
    	    $arrayNutr['NRREMESSA'] = $cdRemessa;

    	    $result = $this->realizarIntegracao($arrayNutr, $url);
    	   
    	    $this->atualizaStatusRemessa($cdRemessa, $nrorgint);
    	    $this->inserirMensagemErro($result['MENSAGEM'], $cdRemessa);
    	    return $result;
    	    
    	}catch(\Exception $e){
	        $atualizacao = $this->inserirMensagemErro($e->getMessage(), $cdRemessa);
	        if ($atualizacao !== true) {
                $response->setError(new Error($atualizacao, 1));
                return false;
            }
            throw new \Exception($e->getMessage());
        }
	}
	
	private function listarParamgeral(){
	    try{
            $query = "SELECT CDPROTEINA, CDLIPIDIOS, CDCARBOHIDRA, CDALCOOL, CDPROTANIMAL, 
                             CDPROTVEGETA, CDACUCARES, CDACIDGRAXOS, CDCALORIAS 
                      FROM PARAMGERAL 
                      WHERE NRORG =:NRORG";
            $stmt = $this->connection->prepare($query);
            $params = array(
                "NRORG"  => $this->nrOrg
            );
            $stmt->execute($params);
            $paramgeral = $stmt->fetchAll(\PDO::FETCH_ASSOC);
            return $paramgeral;
	    }catch(\Exception $e){
	        throw new \Exception($e->getMessage());
	    }
	}
	
	private function excluiDir($dir){
        if ($dd = opendir($dir)) {
            while (false !== ($arq = readdir($dd))) {
                if($arq != "." && $arq != ".."){
                    $path = "$dir/$arq";
                    if(is_dir($path)){
                        self::excluiDir($path);
                    }elseif(is_file($path)){
                        unlink($path);
                    }
                }
            }
            closedir($dd);
        }
        rmdir($dir);
    }

    private function verificarPendencias($rows,  $request, $response){
        $orgPendentes = [];
        $newRows = [];
        
        foreach ($rows as $row) {
            if($row['PENDENTE'] == 'S') continue;
            $remessas = $this->listarPendencias($row['NRORGINT']);
            if(!empty($remessas)){
                try{
                    foreach($remessas as $remessa){
                        $nrremessa = $remessa['NRREMESSA'];
                        $nrorgint = $row['NRORGINT'];
                        $produtosRemessa = $this->listarProdutos($nrremessa, $nrorgint);

                        $arrayProdutos = $this->montarArrayProdutos($produtosRemessa, $nrorgint, $nrremessa, false);

                        $url = $row['URLINTEGRAORG'].$this->rotaReenvia;
                        $this->realizarIntegracao($arrayProdutos, $url);

                        $this->atualizaStatusRemessa($nrremessa, $nrorgint);
                        $this->atualizaDtUltatu($nrremessa, $nrorgint);
                    }
    	        }catch(\Exception $e){
    	            $msg = $this->trataErros($request, $response, $e->getMessage(), $nrremessa, $row);
                    $atualizacao = $this->inserirMensagemErro($msg, $nrremessa);
                    $this->atualizaDtUltatu($nrremessa, $nrorgint);
                    if ($atualizacao !== true) {
                        $response->setError(new Error($atualizacao, 1));
                        return false;
                    }

                    $row['PENDENTE'] = 'S';
                    array_push($orgPendentes,
                    [ "NMORG"   => $row['NMORGANIZACAO'],
                      "ERROR"   => $msg
                    ]);
                
                    
    	        }
    	    }
    	    array_push($newRows, $row);
        }
	    
	    return [$newRows, $orgPendentes];
    }
    
	private function listarPendencias($nrorgint){
	    try {
	        $remessas = "SELECT * FROM REMPRODNUTR
                                  WHERE NRORGINT = :NRORGINT
	                              AND IDSITREMESSA = 'N'
	                              ORDER BY NRREMESSA ASC";
    	    
            $stmt = $this->connection->prepare($remessas);
            $params = array(
                "NRORGINT" => $nrorgint
            );
            $stmt->execute($params);
            $remessas = $stmt->fetchAll(\PDO::FETCH_ASSOC);
            return $remessas;
	    }catch (\Exception $e) {
	        throw new \Exception($e->getMessage());
	    }
	}
	
	private function listarProdutos($nrremessa, $nrorgint){
	    try{
            $produtosRemessa = "SELECT SIT_LOGPROD.NRREMESSARET as NRREMESSA,
                                       PRODUTO.VRPEFATOCORR,
                                       SIT_PRODUTOINT.CDPRODUTOINT,
                                       SIT_PRODUTOINT.CDPRODUTO,
                                       SIT_PRODUTOINT.QTPESOINT,
                                       SIT_PRODUTOINT.IDATIVO
                                  FROM SIT_LOGPROD
                                  LEFT JOIN SIT_PRODUTOINT ON SIT_PRODUTOINT.CDPRODUTOINT = SIT_LOGPROD.CDPRODUTO
                                                           AND SIT_PRODUTOINT.NRREMESSA = SIT_LOGPROD.NRREMESSARET    
                                                           AND SIT_PRODUTOINT.NRORGINT = SIT_LOGPROD.NRORGINT    
                                  LEFT JOIN PRODUTO ON PRODUTO.CDPRODUTO = SIT_LOGPROD.CDPRODUTO
                                 WHERE SIT_LOGPROD.NRORGINT = :NRORGINT
                                   AND SIT_LOGPROD.NRREMESSA = :NRREMESSA
                                   AND SIT_LOGPROD.DSOBSERVACAO IS NULL";
            $stmt = $this->connection->prepare($produtosRemessa);
            $params = array(
                "NRORGINT"  => $nrorgint,
                "NRREMESSA" => $nrremessa
            );
            $stmt->execute($params);
            $produtosRemessa = $stmt->fetchAll(\PDO::FETCH_ASSOC);
            return $produtosRemessa;
	    }catch(\Exception $e){
	        throw new \Exception($e->getMessage());
	    }
	}

	private function gerarCdRemessa(){
        $procedure = new StoredProcedure($this->connection, 'NOVO_CODIGO');
        $procedure->addParam(new Param('P_CONTADOR', Param::PARAM_INPUT, 'REMPRODNUTR', Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SEQUENCIAL', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 20));
        $procedure->addParam(new Param('P_QTDE', Param::PARAM_INPUT, 1 , Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $this->nrOrg, Param::PARAM_TYPE_STR));
        $cdRemessa = $procedure->execute();
        return $cdRemessa['P_SEQUENCIAL'];
	}
	
	private function inserirRemessa($cdRemessa, $nrorgint){
	    try{
            $insereRemessa = "INSERT INTO REMPRODNUTR (NRREMESSA, DTENVIO, IDSITREMESSA, NRORGINT, IDTPREMESSA)
                                   VALUES (:NRREMESSA, :DTENVIO, :IDSITREMESSA, :NRORGINT, :IDTPREMESSA)";
            $stmt = $this->connection->prepare($insereRemessa);
            $params = array(
                "NRREMESSA"    => $cdRemessa,
                "DTENVIO"      => date('d/m/Y H:i:s', time()),
                "IDSITREMESSA" => 'N',
                "NRORGINT"     => $nrorgint,
                "IDTPREMESSA"  => 'R'
            );
	        $stmt->execute($params);
            
            return true;
	    }catch(\Exception $e){
	        return $e->getMessage();
	        //throw new \Exception($e->getMessage());
	    }
	}
	
	private function listarProdutosLiberados($nrorgint){
	    try {
	        $produtosLiberados = "SELECT MIN(SP.NRREMESSA) NRREMESSA, SP.NRORGINT, SP.CDPRODUTOINT, SP.QTPESOINT, SP.CDPRODUTO, SP.IDATIVO, PRODUTO.VRPEFATOCORR
                                      FROM SIT_PRODUTOINT SP
                                      LEFT JOIN PRODUTO ON PRODUTO.CDPRODUTO = SP.CDPRODUTO
                                     WHERE SP.DTLIBERACAO IS NOT NULL
                                       AND SP.DTENVIO IS NULL
                                       AND SP.NRORGINT = :NRORGINT
                                       AND ROWNUM <= 100
                                    GROUP BY SP.NRORGINT, SP.CDPRODUTOINT, SP.QTPESOINT, SP.CDPRODUTO, SP.IDATIVO, PRODUTO.VRPEFATOCORR
                                     ORDER BY SP.CDPRODUTOINT";
                                     
                                     
                                     
	        $stmt = $this->connection->prepare($produtosLiberados);
            $params = array(
                "NRORGINT" => $nrorgint
            );
            $stmt->execute($params);
            $produtosLiberados = $stmt->fetchAll(\PDO::FETCH_ASSOC);
	        return $produtosLiberados;
            
	    } catch (\Exception $e) {
	        throw new \Exception($e->getMessage());
	    }
	}
	
	private function montarArrayProdutos($produtos, $nrorgint, $cdRemessa, $envio){
        $infOrganizacao = $this->listarInfOrganizacao($nrorgint);
        $produtosArray = [];
        if(empty($produtos)){
            $produtosArray = [
                "NRORGINT"   => $nrorgint,
                "NRREMESSA"  => $cdRemessa,
                "QTPRODREC"  => 0,
                "QTPRODRET"  => 0,
                "IDCONSNUT"  => $infOrganizacao[0]['IDCONSNUT'],
                "IDFATCORR"  => $infOrganizacao[0]['IDFATCORR'],
                "PRODUTOS"   => []
            ];
        }else{
            $produtosArray = [
                "NRORGINT"   => $nrorgint,
                "NRREMESSA"  => $cdRemessa,
                "QTPRODREC"  => 0,
                "QTPRODRET"  => count($produtos),
                "IDCONSNUT"  => $infOrganizacao[0]['IDCONSNUT'],
                "IDFATCORR"  => $infOrganizacao[0]['IDFATCORR'],
                "PRODUTOS"   => []
            ];

            foreach($produtos as $produto){
                if($infOrganizacao[0]['IDCONSNUT'] === 'S'){
                    $informacoes = $this->obterInformacoesNutricionais($produto);
                    $cabecalho = $this->preencherCabecalho($produto, $informacoes);
                }else{
                    $cabecalho = $this->preencherCabecalho($produto, []);
                }
                array_push($produtosArray['PRODUTOS'], $cabecalho);
                $this->insereLogProd($produto['CDPRODUTOINT'], $nrorgint, $cdRemessa, $envio, $produto['IDATIVO'], $produto['NRREMESSA']);
                $this->atualizarProdutoInt($produto['CDPRODUTOINT'], $nrorgint, $produto['NRREMESSA'], $envio);
            }
        }
        return $produtosArray;
	}
	
	private function listarInfOrganizacao($nrorgint){
	    try {
	        $informacoes = "SELECT IDCONSNUT, IDFATCORR
                              FROM SIT_CONTRATO 
                             WHERE NRORGINT = :NRORGINT";
            $stmt = $this->connection->prepare($informacoes);
            $params = array(
                "NRORGINT" => $nrorgint
            );
            $stmt->execute($params);
            $informacoes = $stmt->fetchAll(\PDO::FETCH_ASSOC);
            return $informacoes;
            
	    } catch (\Exception $e) {
	        throw new \Exception($e->getMessage());
	    }
	}
	
	private function obterInformacoesNutricionais($produto){
	    try{
            $infoNutricional = "SELECT NUTRIENTE.CDGRUPNUTR, GRUPNUTR.NMGRUPNUTR, NUTRPROD.CDNUTRIENTE, 
                                        NUTRIENTE.NMNUTRIENTE, NUTRIENTE.NMREDUZIDO, NUTRPROD.DTULTATUNUPR, 
                                        NUTRPROD.DSFONTNUTR, NUTRIENTE.SGUNIDADE, NUTRIENTE.IDCALORIAS, 
                                        NUTRIENTE.IDNUTRRELEV, NUTRIENTE.VRFATORNUTR, NUTRPROD.QTNUTRPROD, NUTRIENTE.IDTIPONUTR,
                                        NUTRIENTE.NRORDEMNUTR
                                  FROM NUTRPROD
                                  INNER JOIN NUTRIENTE ON NUTRPROD.CDNUTRIENTE = NUTRIENTE.CDNUTRIENTE
                                  INNER JOIN PRODUTO ON PRODUTO.CDPRODUTO = NUTRPROD.CDPRODUTO
                                  LEFT JOIN GRUPNUTR ON GRUPNUTR.CDGRUPNUTR = NUTRIENTE.CDGRUPNUTR
                                 WHERE NUTRPROD.CDPRODUTO = :CDPRODUTO";
            $stmt = $this->connection->prepare($infoNutricional);
            $params = array(
                "CDPRODUTO" => $produto['CDPRODUTO']
            );
            $stmt->execute($params);
            $infoNutricionais = $stmt->fetchAll(\PDO::FETCH_ASSOC);
            
            $informacoes = [];
            if(!empty($infoNutricionais)){
                foreach($infoNutricionais as $infoNutricional){
                    array_push($informacoes,[
                            "CDGRUPNUTR"   => $infoNutricional['CDGRUPNUTR'],
                            "NMGRUPNUTR"   => $infoNutricional['NMGRUPNUTR'],
                            "CDNUTRIENTE"  => $infoNutricional['CDNUTRIENTE'],
                            "NMNUTRIENTE"  => $infoNutricional['NMNUTRIENTE'],
                            "NMREDUZIDO"   => $infoNutricional["NMREDUZIDO"],
                            "DTULTATUNUPR" => $infoNutricional['DTULTATUNUPR'],
                            "DSFONTNUTR"   => $infoNutricional['DSFONTNUTR'],
                            "SGUNIDADE"    => $infoNutricional['SGUNIDADE'],
                            "IDCALORIAS"   => $infoNutricional['IDCALORIAS'],
                            "IDNUTRRELEV"  => $infoNutricional['IDNUTRRELEV'],
                            "VRFATORNUTR"  => $infoNutricional['VRFATORNUTR'],
                            "QTNUTRPROD"   => $infoNutricional['QTNUTRPROD'],
                            "IDTIPONUTR"   => $infoNutricional['IDTIPONUTR'],
                            "NRORDEMNUTR"  => $infoNutricional['NRORDEMNUTR']
                    ]);   
                }
            }
            return $informacoes;
	    }catch(\Exception $e){
	        throw new \Exception($e->getMessage());
	    }
	}
	
	private function preencherCabecalho($produto, $informacoes){
	    $cabecalhoProduto = [
            "NRREMESSA"    => $produto['NRREMESSA'], 
            "CDPRODUTO"    => $produto['CDPRODUTOINT'],
            "VRPESOUNID"   => $produto['QTPESOINT'],
            "VRPEFATOCORR" => $produto['VRPEFATOCORR'] == null ? 0 : $produto['VRPEFATOCORR'],
            "IDATIVO"      => $produto['IDATIVO'],
            $informacoes
        ];
        return $cabecalhoProduto;
	}
	
	private function atualizarProdutoInt($cdprodutoint, $nrorgint, $nrremessa, $envio){
	    try{
	        if (!$envio) {
	            $condicao = " AND NRREMESSA = $nrremessa";
	        } else {
	            $condicao = ' AND DTENVIO IS NULL';
	        }
	        
    	    $produto = "UPDATE SIT_PRODUTOINT
                           SET DTENVIO = :DTENVIO,
                               DTULTATU = :DTULTATU,
                               CDOPERULTATU = :CDOPERULTATU
                         WHERE DTLIBERACAO IS NOT NULL
                           $condicao 
                           AND CDPRODUTOINT = :CDPRODUTOINT
                           AND NRORGINT = :NRORGINT";
            
            $stmt = $this->connection->prepare($produto);
            $params = array(
                "DTENVIO"      => date('d/m/Y H:i:s', time()),
                "DTULTATU"     => date('d/m/Y H:i:s', time()),
                "CDOPERULTATU" => $this->cdOperador,
                "CDPRODUTOINT" => $cdprodutoint,
                "NRORGINT"     => $nrorgint
            );
            $stmt->execute($params);
	    }catch(\Exception $e){
	        throw new \Exception($e->getMessage());
	    }
	}
	
	private function insereLogProd($cdprodutoint, $nrorgint, $cdRemessa, $envio, $idTemNutriente, $remessaRet = null){
	    if($envio == true){
    	    try{
        	    $remessa = "INSERT INTO SIT_LOGPROD (NRREMESSA, CDPRODUTO, DTENVIO, NRORG, NRORGINT, IDINFONUTRI, NRREMESSARET)
                                VALUES (:NRREMESSA, :CDPRODUTO, :DTENVIO, :NRORG, :NRORGINT, :IDINFONUTRI, :NRREMESSARET)";
                $stmt = $this->connection->prepare($remessa);
                $params = array(
                    "NRREMESSA"    => $cdRemessa,
                    "CDPRODUTO"    => $cdprodutoint,
                    "DTENVIO"      => date('d/m/Y H:i:s', time()),
                    "NRORG"        => $this->nrOrg,
                    "NRORGINT"     => $nrorgint,
                    "IDINFONUTRI"  => $idTemNutriente,
                    "NRREMESSARET" => $remessaRet
                );
                $stmt->execute($params);
    	    }catch(\Exception $e){
    	        throw new \Exception($e->getMessage());
    	    }
	    }
	}
	
	private function realizarIntegracao($array, $url){
	    try{
	        
	        $response = '';
	        $result = '';
	        
            $nrorgInt = $array['NRORGINT'];
            $remessa = ltrim($array['NRREMESSA'],'0');
            $data = date('d/m/Y H:i:s', time());
            $data = str_replace(" ", "-", $data);
            $data = str_replace("/", " ", $data);
            $data = str_replace(" ", "", $data);
            $data = str_replace(":", "", $data);
            
            $fileName = $remessa.'_'.$data.'.txt';
            
            if(strpos($url, "preparaAmbiente") !== false){
                $dirRequest = $this->salvaArqServ($array, 'request_'.$fileName, 'Teknutri/Request/'.$nrorgInt);
                $this->salvaDirRemessa($dirRequest, 'request', $array['NRREMESSA'], $nrorgInt);
            }else{
                if(!empty($array['PRODUTOS'])){
                    $dirRequest = $this->salvaArqServ($array, 'request_'.$fileName, 'Teknutri/Request/'.$nrorgInt);
                    $this->salvaDirRemessa($dirRequest, 'request', $array['NRREMESSA'], $nrorgInt);
                }
            }
            $curl = curl_init(); 
            //$url = 'http://192.168.120.34:9091/tecfood-integra/backend/service/index.php/recebeDados';
            curl_setopt($curl, CURLOPT_URL, $url);
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($curl, CURLOPT_HTTPHEADER, array(                                                                          
                    'Content-Type: application/json'));
            curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($array));
            curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($curl, CURLOPT_TIMEOUT, 30);
            curl_setopt($curl, CURLOPT_FAILONERROR, true);
            
            $response = curl_exec($curl);
            $erro = curl_error($curl);
            $status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            curl_close($curl);

            if($response === false) throw new \Exception('URL_NAO_ENCONTRADA'); 
            $result = json_decode(substr($response, 0, -2), true);

            $dirResponse = $this->salvaArqServ($response, 'response_'.$fileName, 'Teknutri/response/'.$nrorgInt);
            
            // if (!$this->logDir) $this->logDir = getcwd();
            
            // //Arquivos Request
            // $caminhoRequest = $this->logDir.DIRECTORY_SEPARATOR.'Request'.DIRECTORY_SEPARATOR;
            // if (!file_exists($caminhoRequest)) {
            //     mkdir($caminhoRequest); 
            // } 
            // $caminhoRequestOrg = $caminhoRequest.$array['NRORGINT'].DIRECTORY_SEPARATOR;
            // if (!file_exists($caminhoRequestOrg)) {
            //     mkdir($caminhoRequestOrg); 
            // } 
            // $caminho = $caminhoRequestOrg.'request_'.$nrorgInt.'_'.$remessa.'_'.$data.'.txt';
            // file_put_contents($caminho, json_encode($array));

            // //Arquivos Response
            // $caminhoResponse = $this->logDir.DIRECTORY_SEPARATOR.'Response'.DIRECTORY_SEPARATOR;
            // if (!file_exists($caminhoResponse)) {
            //     mkdir($caminhoResponse); 
            // }
            // $caminhoResponseOrg = $caminhoResponse.$array['NRORGINT'].DIRECTORY_SEPARATOR;
            // if (!file_exists($caminhoResponseOrg)) {
            //     mkdir($caminhoResponseOrg); 
            // } 
            // $caminho = $caminhoResponseOrg.'response_'.$nrorgInt.'_'.$remessa.'_'.$data.'.txt';
            
            if(strpos($url, "preparaAmbiente") !== false){
                if(is_array($result)){
                    if(!empty($result['PRODUTOS'])){
                        $dirResponse = $this->salvaArqServ($result, 'response_'.$fileName, 'Teknutri/Response/'.$nrorgInt);
                        $this->salvaDirRemessa($dirResponse, 'response', $array['NRREMESSA'], $nrorgInt);
                    }
                    // file_put_contents($caminho, json_encode($result));
                    return $result;
                }else{           
                    $error = 'PREPARA AMBIENTE - ERROR: ' . $result .'                    ||' . $response . ' ||                 ########### STATUS: ' . $status;
                    $dirResponse = $this->salvaArqServ($error, 'response_'.$fileName, 'Teknutri/Response/'.$nrorgInt);
                    $this->salvaDirRemessa($dirResponse, 'response', $array['NRREMESSA'], $nrorgInt);
                    // file_put_contents($caminho,  $error);
                    throw new \Exception($error);
                }
            }else{
                
                if(is_array($result)){
                    if($result['QTPRODRET'] > 0) {
                        foreach ($result['PRODUTOS'] as $produto) {
                            $this->insereRetornoProdInt($result['NRORGINT'], $result['NRREMESSA'], $produto);
                        }
                        $dirResponse = $this->salvaArqServ($result, 'response_'.$fileName, 'Teknutri/Response/'.$nrorgInt);
                        $this->salvaDirRemessa($dirResponse, 'response', $array['NRREMESSA'], $nrorgInt);
                    }
                    
                    // file_put_contents($caminho, json_encode($result));
                    return $result['QTPRODRET'];
                }else{
                    if(strpos($result, "incompatível") !== false) $error = 'ERROR: ' . $result; 
                    else $error = 'ERROR: ' . $result . '                  || ' . $response . ' ||                  ########### STATUS: ' . $status;
                    $dirResponse = $this->salvaArqServ($error, 'response_'.$fileName, 'Teknutri/Response/'.$nrorgInt);
                    $this->salvaDirRemessa($dirResponse, 'response', $array['NRREMESSA'], $nrorgInt);
                    // file_put_contents($caminho, $error);
                    throw new \Exception($error);
                }
                
            }
            
	    }catch(\Exception $e){
	        if(strpos($url, "preparaAmbiente") !== false){
                throw new \Exception( 'PREPARA AMBIENTE - ERROR: ' . $result . '                  || ' . $response . ' ||                  ########### ' . $e->getMessage());
            } else {
                throw new \Exception($e->getMessage());
            }
	            
	    }
    }
    
    private function insereRetornoProdInt($nrorgint, $remessa, $produto){
        $existe = $this->produtoExiste($nrorgint, $remessa, $produto);
        if(!$existe){
            try{
                $insert = "INSERT INTO SIT_PRODUTOINT (NRORGINT, NRREMESSA, CDPRODUTOINT, NMPRODUTOINT, CDUNIDADEINT, QTPESOINT, DSOBSINT, DTRECEBIMENTO, IDREVISAO, NRORG)
                                   VALUES (:NRORGINT, :NRREMESSA, :CDPRODUTOINT, :NMPRODUTOINT, :CDUNIDADEINT, :QTPESOINT, :OBSERVACAO, SYSDATE, :IDREVISAO, :NRORG)";
                $stmt = $this->connection->prepare($insert);
                $params = array(
                    "NRORGINT"     => $nrorgint,
                    "NRREMESSA"    => $remessa,
                    "CDPRODUTOINT" => $produto['CDPRODUTO'],
                    "NMPRODUTOINT" => $produto['NMPRODUTO'],
                    "CDUNIDADEINT" => $produto['SGUNIDADE'],
                    "QTPESOINT"    => $produto['VRPESOUNID'],
                    "OBSERVACAO"   => $produto['OBSERVACAO'],
                    "IDREVISAO"    => $produto['IDALTERANUTR'] == null ? 'N' : $produto['IDALTERANUTR'],
                    "NRORG"        => $this->nrOrg
                );
                $stmt->execute($params);
            }catch(\Exception $e){
    	        throw new \Exception($e->getMessage());
    	    }
        }
    }
    
    private function produtoExiste($nrorgint, $remessa, $produto){
        
        try{
            $existe = false;
            $consulta = "SELECT * FROM SIT_PRODUTOINT 
                         WHERE CDPRODUTOINT = :CDPRODUTOINT
                          AND NRREMESSA = :NRREMESSA 
                          AND NRORGINT = :NRORGINT";
            $stmt = $this->connection->prepare($consulta);
            $params = array(
                "CDPRODUTOINT" => $produto['CDPRODUTO'],
                "NRREMESSA"    => $remessa,
                "NRORGINT"     => $nrorgint
            );
            $stmt->execute($params);
            $produto = $stmt->fetchAll(\PDO::FETCH_ASSOC);
            
            if(!empty($produto)){
                $existe = true;
            }
            return $existe;
        }catch(\Exception $e){
	        throw new \Exception($e->getMessage());
	    }
    }
    
    private function atualizaStatusRemessa($cdRemessa, $nrorgint){
        try{
            $update = "UPDATE REMPRODNUTR
                          SET IDSITREMESSA  = :IDSITREMESSA,
                              DSMENSERRO = NULL
                        WHERE NRREMESSA = :NRREMESSA
                          AND NRORGINT = :NRORGINT";
            $stmt = $this->connection->prepare($update);
            $params = array(
                "IDSITREMESSA" => "S",
                "NRREMESSA" => $cdRemessa,
                "NRORGINT" => $nrorgint
            );
            
            $stmt->execute($params);
        }catch(\Exception $e){
	        throw new \Exception($e->getMessage());
	    }
    }
    
    private function atualizaDtUltatu($cdRemessa,$nrorgint){
        try{    
            $update = "UPDATE REMPRODNUTR
                          SET DTULTATU  = :DTULTATU
                        WHERE NRREMESSA = :NRREMESSA
                          AND NRORGINT = :NRORGINT";
            $stmt = $this->connection->prepare($update);
            $params = array(
                "DTULTATU" => date('d/m/Y H:i:s', time()),
                "NRREMESSA" => $cdRemessa,
                "NRORGINT"  => $nrorgint
            );
            
            $stmt->execute($params);
        }catch(\Exception $e){
	        throw new \Exception($e->getMessage());
	    }
    }
    
    public function liberaRemessa($request, $response){
        $row = $request->getRow();
        $this->atualizaStatusRemessa($row['NRREMESSA'], $row['NRORGINT']);
        $this->inserirMensagemErro("PREPARAÇÃO DE AMBIENTE MANUAL", $row['NRREMESSA']);
    }
    
    private function inserirMensagemErro($mensagem, $cdRemessa){
        try{
            if (strstr($mensagem, 'ERR_DNS_FAIL') OR strstr($mensagem, 'Route') ) {
                $mensagem = "URL configurado errado! (Exemplo: tekintegra.teknisa.com)";
            }

            $update = "UPDATE REMPRODNUTR
                          SET DSMENSERRO  = :DSMENSERRO
                        WHERE NRREMESSA = :NRREMESSA";
            $stmt = $this->connection->prepare($update);
            $params = array(
                "DSMENSERRO" => $mensagem,
                "NRREMESSA" => $cdRemessa
            );
            $stmt->execute($params);
            return true;
        }catch(\Exception $e){
	        throw new \Exception($e->getMessage());
	    }
    }
    
 
    public function salvaArqServ($data = '', $filename, $subFolder){
        
        if(is_array($data)) $data = json_encode($data);
        
        $tmpFile = sys_get_temp_dir().DIRECTORY_SEPARATOR.'remessa'.uniqid().'.txt';
        file_put_contents($tmpFile, $data);
        $file = file_get_contents($tmpFile);
        $file = base64_encode($file);

        $file = 'data: '.mime_content_type($tmpFile).';base64,'.$file;

        $lastModify = filectime($tmpFile);

        $file = array(
            "name" => $filename, 
            "lastModified" => $lastModify,
            "lastModifiedDate" => '',
            "webkitRelativePath" => '',
            "type" => '',
            "size"  => filesize($tmpFile),
            "b64File" => $file);
            
        $row['TXT']    = array();
        $row['TXT'][0] = $file;

        $this->fileUploader->uploadFilesInRow($row, 'TXT', $subFolder);
        unlink($tmpFile);
        
        $fileServerLink = $row['TXT'][0]['path'];

        return $fileServerLink;
        
    }
    
    private function salvaDirRemessa($dir, $type, $cdRemessa, $nrorgint){
      try{  
            if($type == 'request'){
                $update = "UPDATE REMPRODNUTR
                            SET DSDIRREQUEST  =:DSDIR
                           WHERE NRREMESSA = :NRREMESSA
                             AND NRORGINT = :NRORGINT";   
            }else{
                $update = "UPDATE REMPRODNUTR
                            SET DSDIRRESPONSE  =:DSDIR
                           WHERE NRREMESSA = :NRREMESSA
                             AND NRORGINT = :NRORGINT"; 
            }
          
            $stmt = $this->connection->prepare($update);
            $params = array(
                "DSDIR" => $dir,
                "NRREMESSA" => $cdRemessa,
                "NRORGINT" => $nrorgint
            );
            $stmt->execute($params);
        }catch(\Exception $e){
	        throw new \Exception($e->getMessage());
	    }  
    }
    
    
}